==> forestry_1.9.4-5.0.4.128-sources.jar: forestry/core/items/ItemPipette.java <==
/*******************************************************************************
 * Copyright (c) 2011-2014 SirSengir.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * Various Contributors including, but not limited to:
 * SirSengir (original work), CovertJaguar, Player, Binnie, MysteriousAges
 ******************************************************************************/
package forestry.core.items;

import java.util.List;

import net.minecraft.client.renderer.ItemMeshDefinition;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;

import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidContainerItem;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import forestry.api.core.IModelManager;
import forestry.api.core.IToolPipette;
import forestry.core.config.Constants;
import forestry.core.fluids.PipetteContents;

public class ItemPipette extends ItemForestry implements IToolPipette, IFluidContainerItem {

	public ItemPipette() {
		func_77625_d(1);
		func_77664_n();
	}

	@Override
	public boolean canPipette(ItemStack itemstack) {
		PipetteContents contained = PipetteContents.create(itemstack);
		return contained == null || !contained.isFull();
	}

	@Override
	public int fill(ItemStack itemstack, FluidStack liquid, boolean doFill) {
		PipetteContents contained = PipetteContents.create(itemstack);

		int limit = getCapacity(itemstack);
		int filled;

		if (contained == null) {
			if (liquid.amount > limit) {
				filled = limit;
			} else {
				filled = liquid.amount;
			}

			contained = new PipetteContents(new FluidStack(liquid, filled));
			filled = liquid.amount;
		} else {
			if (contained.getContents().amount >= limit) {
				return 0;
			}
			if (!contained.getContents().isFluidEqual(liquid)) {
				return 0;
			}

			int space = limit - contained.getContents().amount;

			if (liquid.amount > space) {
				filled = space;
			} else {
				filled = liquid.amount;
			}

			contained.getContents().amount += filled;
		}

		if (doFill) {
			NBTTagCompound nbttagcompound = new NBTTagCompound();
			contained.writeToNBT(nbttagcompound);
			itemstack.func_77982_d(nbttagcompound);
			itemstack.func_77964_b(1);
		}

		return filled;
	}

	@Override
	public void func_77624_a(ItemStack itemstack, EntityPlayer player, List<String> list, boolean flag) {
		PipetteContents contained = PipetteContents.create(itemstack);
		if (contained != null) {
			contained.addTooltip(list);
		}
	}

	/* Models */
	@SideOnly(Side.CLIENT)
	public ModelResourceLocation[] models;

	@SideOnly(Side.CLIENT)
	@Override
	public void registerModel(Item item, IModelManager manager) {
		models = new ModelResourceLocation[2];
		models[0] = manager.getModelLocation("pipette.0");
		models[1] = manager.getModelLocation("pipette.1");
		manager.registerVariant(item, new ResourceLocation("forestry:pipette.0"));
		manager.registerVariant(item, new ResourceLocation("forestry:pipette.1"));
		manager.registerItemModel(item, new PipetteMeshDefinition());
	}

	public class PipetteMeshDefinition implements ItemMeshDefinition {

		@Override
		public ModelResourceLocation func_178113_a(ItemStack stack) {
			if (stack.func_77952_i() <= 0) {
				return models[0];
			} else {
				return models[1];
			}
		}

	}

	@Override
	public FluidStack drain(ItemStack pipette, int maxDrain, boolean doDrain) {
		PipetteContents contained = PipetteContents.create(pipette);
		if (contained == null) {
			return null;
		}

		int drained = maxDrain;
		if (contained.getContents().amount < drained) {
			drained = contained.getContents().amount;
		}

		if (doDrain) {
			contained.getContents().amount -= drained;

			if (contained.getContents().amount <= 0) {
				pipette.func_77982_d(null);
				pipette.func_77964_b(0);
			} else {
				NBTTagCompound nbttagcompound = new NBTTagCompound();
				contained.writeToNBT(nbttagcompound);
				pipette.func_77982_d(nbttagcompound);
			}
		}

		return new FluidStack(contained.getContents(), drained);
	}

	@Override
	public FluidStack getFluid(ItemStack container) {
		PipetteContents contained = PipetteContents.create(container);
		if (contained == null) {
			return null;
		}
		return contained.getContents();
	}

	@Override
	public int getCapacity(ItemStack pipette) {
		return Constants.BUCKET_VOLUME;
	}
}

==> forestry_1.9.4-5.0.7.135-sources.jar: forestry/core/items/ItemPipette.java <==
/*******************************************************************************
 * Copyright (c) 2011-2014 SirSengir.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * Various Contributors including, but not limited to:
 * SirSengir (original work), CovertJaguar, Player, Binnie, MysteriousAges
 ******************************************************************************/
package forestry.core.items;

import java.util.List;

import net.minecraft.client.renderer.ItemMeshDefinition;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;

import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidContainerItem;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import forestry.api.core.IModelManager;
import forestry.api.core.IToolPipette;
import forestry.core.fluids.PipetteContents;

public class ItemPipette extends ItemForestry implements IToolPipette, IFluidContainerItem {

	public ItemPipette() {
		func_77625_d(1);
		func_77664_n();
	}

	@Override
	public boolean canPipette(ItemStack itemstack) {
		PipetteContents contained = PipetteContents.create(itemstack);
		return contained == null || !contained.isFull();
	}

	@Override
	public int fill(ItemStack itemstack, FluidStack liquid, boolean doFill) {
		PipetteContents contained = PipetteContents.create(itemstack);

		int limit = getCapacity(itemstack);
		int filled;

		if (contained == null) {
			if (liquid.amount > limit) {
				filled = limit;
			} else {
				filled = liquid.amount;
			}

			contained = new PipetteContents(new FluidStack(liquid, filled));
			filled = liquid.amount;
		} else {
			if (contained.getContents().amount >= limit) {
				return 0;
			}
			if (!contained.getContents().isFluidEqual(liquid)) {
				return 0;
			}

			int space = limit - contained.getContents().amount;

			if (liquid.amount > space) {
				filled = space;
			} else {
				filled = liquid.amount;
			}

			contained.getContents().amount += filled;
		}

		if (doFill) {
			NBTTagCompound nbttagcompound = new NBTTagCompound();
			contained.writeToNBT(nbttagcompound);
			itemstack.func_77982_d(nbttagcompound);
			itemstack.func_77964_b(1);
		}

		return filled;
	}

	@Override
	public void func_77624_a(ItemStack itemstack, EntityPlayer player, List<String> list, boolean flag) {
		PipetteContents contained = PipetteContents.create(itemstack);
		if (contained != null) {
			contained.addTooltip(list);
		}
	}

	/* Models */
	@SideOnly(Side.CLIENT)
	public ModelResourceLocation[] models;

	@SideOnly(Side.CLIENT)
	@Override
	public void registerModel(Item item, IModelManager manager) {
		models = new ModelResourceLocation[2];
		models[0] = manager.getModelLocation("pipette.0");
		models[1] = manager.getModelLocation("pipette.1");
		manager.registerVariant(item, new ResourceLocation("forestry:pipette.0"));
		manager.registerVariant(item, new ResourceLocation("forestry:pipette.1"));
		manager.registerItemModel(item, new PipetteMeshDefinition());
	}

	public class PipetteMeshDefinition implements ItemMeshDefinition {

		@Override
		public ModelResourceLocation func_178113_a(ItemStack stack) {
			if (stack.func_77952_i() <= 0) {
				return models[0];
			} else {
				return models[1];
			}
		}

	}

	@Override
	public FluidStack drain(ItemStack pipette, int maxDrain, boolean doDrain) {
		PipetteContents contained = PipetteContents.create(pipette);
		if (contained == null) {
			return null;
		}

		int drained = maxDrain;
		if (contained.getContents().amount < drained) {
			drained = contained.getContents().amount;
		}

		if (doDrain) {
			contained.getContents().amount -= drained;

			if (contained.getContents().amount <= 0) {
				pipette.func_77982_d(null);
				pipette.func_77964_b(0);
			} else {
				NBTTagCompound nbttagcompound = new NBTTagCompound();
				contained.writeToNBT(nbttagcompound);
				pipette.func_77982_d(nbttagcompound);
			}
		}

		return new FluidStack(contained.getContents(), drained);
	}

	@Override
	public FluidStack getFluid(ItemStack container) {
		PipetteContents contained = PipetteContents.create(container);
		if (contained == null) {
			return null;
		}
		return contained.getContents();
	}

	@Override
	public int getCapacity(ItemStack pipette) {
		return Fluid.BUCKET_VOLUME;
	}
}
